#include <stdio.h>
#include <string.h>

#include <c-flags.h>

#include <recorder/config.h>
#include <recorder/config_defaults.h>

PS2000_RANGE parse_range(char* range) {
  if (strcmp(range, "10mV") == 0)
    return PS2000_10MV;
  if (strcmp(range, "20mV") == 0)
    return PS2000_20MV;
  if (strcmp(range, "50mV") == 0)
    return PS2000_50MV;
  if (strcmp(range, "100mV") == 0)
    return PS2000_100MV;
  if (strcmp(range, "200mV") == 0)
    return PS2000_200MV;
  if (strcmp(range, "500mV") == 0)
    return PS2000_500MV;
  if (strcmp(range, "1V") == 0)
    return PS2000_1V;
  if (strcmp(range, "2V") == 0)
    return PS2000_2V;
  if (strcmp(range, "5V") == 0)
    return PS2000_5V;
  if (strcmp(range, "10V") == 0)
    return PS2000_10V;
  if (strcmp(range, "20V") == 0)
    return PS2000_20V;
  if (strcmp(range, "50V") == 0)
    return PS2000_50V;
  return PS2000_MAX_RANGES;
}

PS2000_TIME_UNITS parse_units(char* unit) {
  if (strcmp(unit, "fs") == 0)
    return PS2000_FS;
  if (strcmp(unit, "ps") == 0)
    return PS2000_PS;
  if (strcmp(unit, "ns") == 0)
    return PS2000_NS;
  if (strcmp(unit, "us") == 0)
    return PS2000_US;
  if (strcmp(unit, "ms") == 0)
    return PS2000_MS;
  if (strcmp(unit, "s") == 0)
    return PS2000_S;
  return PS2000_MAX_TIME_UNITS;
}

config* get_config(int argc, char** argv) {
  if (argc > 0)
    c_flags_set_application_name(argv[0]);

  char** range = c_flag_string("range", "r", "voltage range", DEFAULT_RANGE);
  bool* dc_mode = c_flag_bool("dc", "dc", "dc mode", DEFAULT_DC_MODE);
  bool* iq_mode = c_flag_bool("iq_mode", "iq", "IQ mode", DEFAULT_IQ_MODE);
  uint32_t* buffersize = c_flag_uint32("buffersize", "b", "buffersize", DEFAULT_BUFFERSIZE);
  uint32_t* max_buffersize = c_flag_uint32("max_buffersize", "mb", "max buffersize the driver stores", DEFAULT_MAX_BUFFERSIZE);
  uint32_t* sample_interval = c_flag_uint32("sample_interval", "s", "sample interval", DEFAULT_SAMPLE_INTERVAL);
  char** time_units = c_flag_string("time_units", "u", "time units", DEFAULT_TIME_UNITS);

  c_flags_parse(&argc, &argv, true);

  config* cfg = (config*)malloc(sizeof(config));
  cfg->dc_mode = *dc_mode;
  cfg->iq_mode = *iq_mode;
  cfg->buffersize = *buffersize;
  cfg->max_buffersize = *max_buffersize;
  cfg->sample_interval = *sample_interval;

  // validate arguments
  cfg->range = parse_range(*range);
  if (cfg->range == PS2000_MAX_RANGES) {
    fprintf(stderr, "invalid voltage range: %s\n", *range);
    return NULL;
  }

  cfg->time_units = parse_units(*time_units);
  if (cfg->time_units == PS2000_MAX_TIME_UNITS) {
    fprintf(stderr, "invalid time unit: %s\n", *time_units);
    return NULL;
  }

  if (*max_buffersize <= *buffersize) {
    fprintf(stderr, "max buffersize needs to be bigger than buffersize\n");
    return NULL;
  }

  return cfg;
}